doc

  mkaprole.sql

  Create the app_roles table.  This table will be used with the
tsec procedure to set password protected roles during the execution
of an application.

#

drop table app_roles;

create table app_roles
  (ROLE_NAME       VARCHAR2(30) NOT NULL , -- Name of role
   PROGRAM         VARCHAR2(48) NOT NULL ,
     -- TOOL Program associated with role.  Value found in v$session
   PASSWD          VARCHAR2(30) NOT NULL , -- Password for role
   APPLICATION     VARCHAR2(8) NOT NULL ,  -- Application name
     -- Application authorized to use role.  Argument 1 in function login.main.
   ROLE_DESC       VARCHAR2(80) -- Description
  )
tablespace &1
storage  (initial   50K
             next   24K
       maxextents  100
      pctincrease    0
         )
;

comment on table app_roles is
  'Security table for role passwords.  Read only by function tset.srole to extract the passwords.';
comment on column app_roles.role_name is
  'Name of role as it was created.';
comment on column app_roles.program is
  'TOOL program associated with the role.  Value comes from v$session';
comment on column app_roles.passwd is
  'Password for role.  Not encrypted.';
comment on column app_roles.application is
  'Name of application which should be running when role is enabled.';
comment on column app_roles.role_desc is
  'Comment or description of entry.';
