doc  

     File:  mkpwsec.sql

     This script must be executed by the owner of the app_roles table.

     This script creates the tsec package and package body.
The body contains one function:  srole.  The function is used as
an argument to the standard database procedure dbms_session.set_role.
This procedure requires a string of roles to be set.  For example,
suppose a user had been granted the finadmin (financial administrator)
and finpersadmin (financial personnel administrator) roles, and that 
these roles were not set as defaults.  Then the command:

    dbms_session.set_role('finadmin','finpersadmin');

would activate the two roles.  If there were a password (blah) for
the finadmin role, then the command would be:

    dbms_session.set_role(finadmin identified by blah,finpersadmin);

    To avoid coding all the roles and passwords into applications,
the srole function will assemble the string of roles and passwords
for use by dbms_session.set_role.  Function srole is then used
as the argument to set_role thus:


      EXEC DBMS_SESSION.SET_ROLE(tsec.srole('APPLICATION_NAME'));

    Since the function is embedded in the call to set_role, it
will be executed first and will be executed within the database.  
The returned string becomes the argument list for set_role which
also executes within the database.  This means that the only argument
visable outside of the database when the call is made is the
application name required by function srole.  The returned string
of roles and passwords NEVER appears outside of the database,
nor is it ever seen by the application.
      
     The srole function returns a string to enable all roles granted
to the user appropriate for the application when the appropriate tool 
is running an appropriate application, as well as the user's default 
roles (including those granted to PUBLIC).

     Passwords for roles are maintained in APP_ROLES table, which has
the following structure:

 Name            Null?    Type
 --------------- -------- ----
 ROLE_NAME       NOT NULL VARCHAR2(30) -- Name of role
 PROGRAM         NOT NULL VARCHAR2(48)
     -- TOOL Program associated with role.  Value found in v$session
 PASSWD          NOT NULL VARCHAR2(30) -- Password for role
 APPLICATION     NOT NULL VARCHAR2(8)  -- Application name
     -- Application authorized to use role.  Argument 1 in function tsec.srole.
 ROLE_DESC                VARCHAR2(80) -- Description


Logic:

     The function checks V$SESSION to insure the the user is
running the correct TOOL for the role (like 'F45RUN.EXE').  If
the user is not running a registered tool and program, this function 
returns only the default roles.

     Calling format:

      EXEC DBMS_SESSION.SET_ROLE(tsec.srole('APPLICATION_NAME'));

     Returned value (assumes a standard role:  genselect):

     genselect, abc identified by xyz, def identified by uvw, ...

     Where:

          APPLICATION_NAME corresponds to the APPLICATION field
          in the table.  It is not case sensitive.

          abc and def are an example role names, and xyz and uvw
          are example role passwords.

     If the argument is "ALL", the all roles for the user will be
activated.

     When creating users, grant all roles to the user, then alter
the user to make the role with "create session" the default role. 
For example a standard select only role would be GENSELECT.

#


create or replace package tsec is
  function srole (arg1 in varchar2) return varchar2;
end;
/

create or replace package body tsec as
function srole (arg1 in varchar2) return varchar2 is
  role_cmd varchar2(256);
  prog_name varchar2(48);
  i integer;
-- Get the password protected roles then get the user's default role.
  cursor c1 is
    select role_name ||
      decode(a.passwd, null, null, ' IDENTIFIED BY '||a.passwd) result
      from app_roles a, dba_role_privs r,
           v$session v, dba_roles dr
     where upper(a.application) = upper(arg1)
       and a.role_name = r.granted_role
       and a.role_name = dr.role
       and r.grantee = user
       and dr.password_required = 'YES'
       and instr(v.program, a.program) > 0
--       and upper(a.program) = upper(v.program)
       and (instr(upper(nvl(v.module,'NO MODULE')),'SQL') = 0
            or
            upper(a.program) = 'ALL')
       and v.audsid = userenv('SESSIONID')
    union
    select drp.granted_role
      from dba_role_privs drp, dba_roles dr
     where drp.granted_role = dr.role
       and dr.password_required = 'NO'
       and drp.default_role = 'YES'
       and grantee in (user, 'PUBLIC')
    ;
                           
role_rec c1%ROWTYPE;

begin

-- Get the program the user is executing from v$session.
-- The USERENV function returns a unique audit session id for
-- each session.  A session represents a database connection.
-- For example, an VisualBasic application session and a Q+E
-- session would return different rows.
/*
select 
  substr(program, instr(program,'\',-1) + 1) into prog_name
      from v$session where audsid = userenv('SESSIONID');
*/
-- Following query will return no rows if the program in
-- v$session does not match the program in app_roles table
  i := 0;
  open c1;
  LOOP
     FETCH c1 into role_rec;
     EXIT WHEN c1%NOTFOUND;
     i := i+1;
     IF (i = 1) THEN
        role_cmd := role_rec.result;
     ELSE
        role_cmd := role_cmd || ', ' || role_rec.result;
     END IF;
  END LOOP;
  CLOSE c1;
  RETURN role_cmd;
end;
end tsec;
/

