doc

   mksysadm.sql

   This script must be executed by user sys.

   Arguments:  1 =  username for sysadm user (password is the same)
               2 =  default tablespace for sysadm user
               3 =  temporary tablespace for sysadm user
               4 =  owner of app_roles table
               5 =  password for owner


   Create the privileges necessary for a system administrator
   role.  Create a system administration user.  This user must
   be able to create and drop users and roles and also update
   the app_roles table.  Access to certain system tables is also
   required.  Privileges required are checked in the user role
   maintenance form and it will not start if the seven privileges
   specified here are not granted.

#
set termout on echo on
spool mksysadm.lis


-- Attempt to drop the role.  Error here is ok.

drop role admin_usr_role;

-- Create the sysadmin role

create role admin_usr_role;

-- Privilege to connect and establish a login session
grant create session to admin_usr_role;

-- Privileges for personal objects
grant create synonym to admin_usr_role;
grant create procedure to admin_usr_role;

-- Privileges to administer users
grant create user to admin_usr_role;
grant drop user to admin_usr_role;
grant alter user to admin_usr_role;

-- Privileges to administer roles
grant create role to admin_usr_role;
grant drop any role to admin_usr_role;
grant alter any role to admin_usr_role;
grant grant any role to admin_usr_role;

-- Privilege to grant privileges to roles or users
grant grant any privilege to admin_usr_role;


-- Create the system administration user, grant the role,
-- set default and temporary tablespaces.

create user &1 identified by &1;

grant admin_usr_role to &1;

alter user &1
  default tablespace &2
  temporary tablespace &3
  default role admin_usr_role
  ;


-- Grant privileges on sys's views to the system administrator
-- not the role.  This user will be executing PL/SQL and that
-- engine does not recognize privileges granted through roles.

grant select on &tab to &1 with grant option

define tab=sys.dba_roles
/

define tab=sys.dba_role_privs
/

define tab=sys.dba_users
/

define tab=sys.dba_sys_privs
/

define tab=sys.dba_tablespaces
/

define tab=sys.system_privilege_map
/

define tab=v_$session
/

-- Create synonyms for the security user

create synonym &1..dba_roles for sys.dba_roles;
create synonym &1..dba_role_privs for sys.dba_role_privs;
create synonym &1..dba_users for sys.dba_users;
create synonym &1..dba_sys_privs for sys.dba_sys_privs;
create synonym &1..dba_tablespaces for sys.dba_tablespaces;
create synonym &1..system_privilege_map for sys.system_privilege_map;
create synonym &1..v$session for sys.v_$session;

-- Connect as the app_roles table owner and grant privs to security

connect &4/&5

grant select, insert, update, delete on app_roles to &1
  with grant option;

-- Connect as the sysadm user and create a private synonym
-- for the role security table and the sys tables.  Cannot
-- guarantee that public synonyms exist.

connect &1/&1

create synonym app_roles for &4..app_roles;

spool off
set echo off
prompt
prompt SPOOL FILE FOR MKSYSADM IS:  mksysadm.lis
prompt
